@@ -41,8 +41,8 @@ def get_invalid_names() -> List[str]:
     invalid_names = [ 'send', 'rcv', 'len', 'round', 'list', 'filter', 'self', 'jid',
                       'datetime', 'random', 'numpy', 'json', 'spade', 'copy', 'uuid',
                       'get_json_from_spade_message', 'get_spade_message', 'logger',
-                      'any', 'sys', 'limit_number', 'BackupBehaviour', 'backup_url', 
-                      'backup_period', 'backup_delay' ]
+                      'any', 'sys', 'limit_number', 'int', 'BackupBehaviour', 
+                      'backup_url', 'backup_period', 'backup_delay' ]
     invalid_names.extend(keyword.kwlist)
     return invalid_names
 
